"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const instance_1 = require("./instance");
function Repeater(options) {
    if (!options) {
        options = {};
    }
    const data = {
        errorCatcher: options.errorCatcher,
        heartbeatInterval: options.heartbeatInterval,
        tasks: [],
    };
    return target => {
        /* Extend class prototype to start taskmaster in constructor */
        return class extends target.prototype.constructor {
            constructor(...args) {
                super(...args);
                const taskmaster = new instance_1.RepeaterInstance(data);
                taskmaster.start();
                taskmaster.setOrigin(this);
                /* Add tasks defined by @Task and stored in prototype */
                const tasks = target.prototype[instance_1.TASKMASTER_TASKS];
                if (tasks) {
                    for (const taskName of Object.keys(tasks)) {
                        const task = tasks[taskName];
                        taskmaster.addTask(taskName, task.function, task.interval, task.payload || []);
                    }
                }
                target.prototype[instance_1.TASKMASTER_INSTANCE_PROP] = taskmaster;
            }
        };
    };
}
exports.Repeater = Repeater;
function RepeaterTask(first, second) {
    const interval = typeof first === 'number' ? first : 0;
    const payload = typeof first === 'object' ? first : second;
    if (payload && !Array.isArray(payload)) {
        throw new Error(`Task payload must be an array of arguments`);
    }
    return (target, propertyKey, propertyDescriptor) => {
        const proto = target.constructor.prototype;
        if (!proto[instance_1.TASKMASTER_TASKS]) {
            proto[instance_1.TASKMASTER_TASKS] = {};
        }
        proto[instance_1.TASKMASTER_TASKS][propertyKey] = {
            function: propertyDescriptor.value,
            interval,
            payload,
        };
    };
}
exports.RepeaterTask = RepeaterTask;
//# sourceMappingURL=decorators.js.map