"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
exports.TASKMASTER_TASKS = '__taskmaster_tasks';
exports.TASKMASTER_INSTANCE_PROP = '__taskmaster_instance';
function getRepeaterInstance(object) {
    return object[exports.TASKMASTER_INSTANCE_PROP];
}
exports.getRepeaterInstance = getRepeaterInstance;
function getRepeaterTasks(object) {
    return object[exports.TASKMASTER_TASKS];
}
exports.getRepeaterTasks = getRepeaterTasks;
class RepeaterInstance {
    constructor(data) {
        this.isBusy = false;
        this.heartbeatInterval = data.heartbeatInterval || 100;
        this.tasks = {};
        if (typeof data.errorCatcher === 'function') {
            this.errorCatcher = data.errorCatcher;
        }
        else {
            /* tslint:disable-next-line:no-console */
            this.errorCatcher = (error, fnName) => console.error(`[${fnName}] ${error.stack}`);
        }
    }
    setOrigin(origin) {
        this.origin = origin;
    }
    start() {
        this.isBusy = true;
        setImmediate(() => __awaiter(this, void 0, void 0, function* () {
            yield this.processTasks();
            this.isBusy = false;
        }));
        this.intervalId = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            if (!this.isBusy) {
                this.isBusy = true;
                yield this.processTasks();
                this.isBusy = false;
            }
        }), this.heartbeatInterval);
    }
    stop() {
        clearInterval(this.intervalId);
        this.intervalId = 0;
    }
    addTask(name, task, interval, payload) {
        if (this.tasks[name]) {
            throw new Error(`Task duplicate: ${name}`);
        }
        if (interval && interval < this.heartbeatInterval) {
            throw new Error(`Task interval can't be shorter than heartbeat interval`);
        }
        this.tasks[name] = {
            function: utils_1.throttler(interval || 0, task.bind(this.origin), `${this.origin.constructor.name}.${task.name}`, this.errorCatcher.bind(this.origin)),
            payload,
        };
        return this;
    }
    processTasks() {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(Object.keys(this.tasks).map((taskName) => __awaiter(this, void 0, void 0, function* () {
                const task = this.tasks[taskName];
                yield task.function.apply(this.origin, task.payload);
            })));
        });
    }
}
exports.RepeaterInstance = RepeaterInstance;
//# sourceMappingURL=instance.js.map